@@ -515,6 +515,7 @@ auparse_state_t *auparse_init(ausource_t source, const void *b)
 	au->escape_mode = AUPARSE_ESC_TTY;
 	au->message_mode = MSG_QUIET;
 	au->debug_message = DBG_NO;
+	init_classify(&au->cl_data);
 
 	return au;
 bad_exit:
@@ -612,7 +613,6 @@ void auparse_feed_age_events(auparse_state_t *au)
 
 void auparse_set_escape_mode(auparse_state_t *au, auparse_esc_t mode)
 {
-	set_escape_mode(mode);
 	if (au == NULL)
 		return;
 	au->escape_mode = mode;
@@ -886,11 +886,8 @@ void ausearch_clear(auparse_state_t *au)
 	au->search_where = AUSEARCH_STOP_EVENT;
 }
 
-void auparse_destroy(auparse_state_t *au)
+static void auparse_destroy_common(auparse_state_t *au)
 {
-	aulookup_destroy_uid_list();
-	aulookup_destroy_gid_list();
-
 	if (au == NULL)
 		return;
 
@@ -920,11 +917,29 @@ void auparse_destroy(auparse_state_t *au)
 		au->in = NULL;
 	}
 	free_interpretation_list();
+	clear_classify(&au->cl_data);
 	au_lol_clear(au->au_lo, 0);
 	free(au->au_lo);
 	free(au);
 }
 
+void auparse_destroy(auparse_state_t *au)
+{
+	aulookup_destroy_uid_list();
+	aulookup_destroy_gid_list();
+
+	auparse_destroy_common(au);
+}
+
+void auparse_destroy_ext(auparse_state_t *au, auparse_destroy_what_t what)
+{
+	if (what == AUPARSE_DESTROY_COMMON)
+		auparse_destroy_common(au);
+	else if (what == AUPARSE_DESTROY_ALL)
+		auparse_destroy(au);
+	return;
+}
+
 /* alloc a new buffer, cur_buf which contains a null terminated line
  * without a newline (note, this implies the line may be empty (strlen == 0)) if
  * successfully read a blank line (e.g. containing only a single newline).
@@ -1486,6 +1501,7 @@ static int au_auparse_next_event(auparse_state_t *au)
 // Brute force go to next event. Returns < 0 on error, 0 no data, > 0 success
 int auparse_next_event(auparse_state_t *au)
 {
+	clear_classify(&au->cl_data);
 	return au_auparse_next_event(au);
 }
 
@@ -1577,6 +1593,15 @@ unsigned int auparse_get_num_records(auparse_state_t *au)
 	return aup_list_get_cnt(au->le);
 }
 
+unsigned int auparse_get_record_num(auparse_state_t *au)
+{
+	rnode *r = aup_list_get_cur(au->le);
+	if (r) 
+		return r->item;
+
+	return 0;
+}
+
 
 /* Functions that traverse records in the same event */
 int auparse_first_record(auparse_state_t *au)
@@ -1782,8 +1807,10 @@ const char *auparse_find_field_next(auparse_state_t *au)
 			if (nvlist_find_name(&r->nv, au->find_field))
 				return nvlist_get_cur_val(&r->nv);
 			r = aup_list_next(au->le);
-			if (r)
+			if (r) {
 				aup_list_first_field(au->le);
+				load_interpretation_list(r->interp);
+			}
 		}
 	}
 	return NULL;
@@ -1791,6 +1818,33 @@ const char *auparse_find_field_next(auparse_state_t *au)
 
 
 /* Accessors to field data */
+unsigned int auparse_get_field_num(auparse_state_t *au)
+{
+	rnode *r = aup_list_get_cur(au->le);
+	if (r) {
+		nvnode *n = nvlist_get_cur(&r->nv);
+		if (n)
+			return n->item;
+	}
+	return 0;
+}
+
+int auparse_goto_field_num(auparse_state_t *au, unsigned int num)
+{
+	if (au->le == NULL)
+		return 0;
+
+	rnode *r = aup_list_get_cur(au->le);
+	if (r) {
+		if (num >= r->nv.cnt)
+			return 0;
+
+		if ((nvlist_goto_rec(&r->nv, num)))
+			return 1;
+	}
+	return 0;
+}
+
 const char *auparse_get_field_name(auparse_state_t *au)
 {
 	if (au->le->e.sec) {
@@ -1842,7 +1896,7 @@ const char *auparse_interpret_field(auparse_state_t *au)
         if (au->le->e.sec) {
                 rnode *r = aup_list_get_cur(au->le);
                 if (r)
-                        return nvlist_interp_cur_val(r);
+                        return nvlist_interp_cur_val(r, au->escape_mode);
         }
 	return NULL;
 }
