@@ -22,8 +22,7 @@
 */
 
 #include "config.h"
-#include "nvlist.h"
-#include "nvpair.h"
+#include "lru.h"
 #include "libaudit.h"
 #include "internal.h"
 #include "interpret.h"
@@ -120,8 +119,7 @@ typedef enum { S_UNSET=-1, S_FAILED, S_SUCCESS } success_t;
 static char *print_escaped(const char *val);
 static const char *print_signals(const char *val, unsigned int base);
 
-// FIXME: move next two declarations to auparse_state_t
-static auparse_esc_t escape_mode = AUPARSE_ESC_TTY;
+// FIXME: move next declaration to auparse_state_t
 static nvlist il;  // Interpretations list
 
 /*
@@ -240,7 +238,8 @@ static void shell_quote_escape(const char *s, char *dest, unsigned int len)
 }
 
 /* This should return the count of what needs escaping */
-static unsigned int need_escaping(const char *s, unsigned int len)
+static unsigned int need_escaping(const char *s, unsigned int len,
+	auparse_esc_t escape_mode)
 {
 	switch (escape_mode)
 	{
@@ -251,12 +250,13 @@ static unsigned int need_escaping(const char *s, unsigned int len)
 		case AUPARSE_ESC_SHELL:
 			return need_shell_escape(s, len);
 		case AUPARSE_ESC_SHELL_QUOTE:
-			return need_shell_quote_escape(s, len);;
+			return need_shell_quote_escape(s, len);
 	}
 	return 0;
 }
 
-static void escape(const char *s, char *dest, unsigned int len)
+static void escape(const char *s, char *dest, unsigned int len,
+	auparse_esc_t escape_mode)
 {
 	switch (escape_mode)
 	{
@@ -271,7 +271,7 @@ static void escape(const char *s, char *dest, unsigned int len)
 	}
 }
 
-static void key_escape(char *orig, char *dest)
+static void key_escape(char *orig, char *dest, auparse_esc_t escape_mode)
 {
 	const char *optr = orig;
 	char *str, *dptr = dest, tmp;
@@ -284,11 +284,11 @@ static void key_escape(char *orig, char *dest)
 		klen = str - optr;
 		tmp = *str;
 		*str = 0;
-		cnt = need_escaping(optr, klen);
+		cnt = need_escaping(optr, klen, escape_mode);
 		if (cnt == 0)
 			dptr = stpcpy(dptr, optr);
 		else {
-			escape(optr, dptr, klen);
+			escape(optr, dptr, klen, escape_mode);
 			dptr = strchr(dest, 0);
 			if (dptr == NULL)
 				return; // Something is really messed up
@@ -305,14 +305,6 @@ static void key_escape(char *orig, char *dest)
 	}
 }
 
-int set_escape_mode(auparse_esc_t mode)
-{
-	if (mode < 0 || mode > AUPARSE_ESC_SHELL_QUOTE)
-		return 1;
-	escape_mode = mode;
-	return 0;
-}
-
 static int is_hex_string(const char *str)
 {
 	while (*str) {
@@ -489,38 +481,44 @@ static const char *aulookup_success(int s)
 	}
 }
 
-static nvpair uid_nvl;
-static int uid_list_created=0;
+static Queue *uid_cache = NULL;
+static int uid_cache_created = 0;
 static const char *aulookup_uid(uid_t uid, char *buf, size_t size)
 {
 	char *name = NULL;
-	int rc;
+	unsigned int key;
+	QNode *q_node;
 
 	if (uid == -1) {
 		snprintf(buf, size, "unset");
 		return buf;
+	} else if (uid == 0) {
+		snprintf(buf, size, "root");
+		return buf;
 	}
 
 	// Check the cache first
-	if (uid_list_created == 0) {
-		nvpair_create(&uid_nvl);
-		nvpair_clear(&uid_nvl);
-		uid_list_created = 1;
+	if (uid_cache_created == 0) {
+		uid_cache = init_lru(19, NULL, "uid");
+		uid_cache_created = 1;
 	}
-	rc = nvpair_find_val(&uid_nvl, uid);
-	if (rc) {
-		name = uid_nvl.cur->name;
-	} else {
-		// This getpw use is OK because its for protocol 1 compatibility
-		// Add it to cache
-		struct passwd *pw;
-		pw = getpwuid(uid);
-		if (pw) {
-			nvpnode nv;
-			nv.name = strdup(pw->pw_name);
-			nv.val = uid;
-			nvpair_append(&uid_nvl, &nv);
-			name = uid_nvl.cur->name;
+	key = compute_subject_key(uid_cache, uid);
+	q_node = check_lru_cache(uid_cache, key);
+	if (q_node) {
+		if (q_node->id == uid)
+			name = q_node->str;
+		else {
+			// This getpw use is OK because its for protocol 1
+			// compatibility.  Add it to cache.
+			struct passwd *pw;
+			lru_evict(uid_cache, key);
+			q_node = check_lru_cache(uid_cache, key);
+			pw = getpwuid(uid);
+			if (pw) {
+				q_node->str = strdup(pw->pw_name);
+				q_node->id = uid;
+				name = q_node->str;
+			}
 		}
 	}
 	if (name != NULL)
@@ -532,44 +530,50 @@ static const char *aulookup_uid(uid_t uid, char *buf, size_t size)
 
 void aulookup_destroy_uid_list(void)
 {
-	if (uid_list_created == 0)
+	if (uid_cache_created == 0)
 		return;
 
-	nvpair_clear(&uid_nvl); 
-	uid_list_created = 0;
+	destroy_lru(uid_cache); 
+	uid_cache_created = 0;
 }
 
-static nvpair gid_nvl;
-static int gid_list_created=0;
+static Queue *gid_cache = NULL;
+static int gid_cache_created = 0;
 static const char *aulookup_gid(gid_t gid, char *buf, size_t size)
 {
 	char *name = NULL;
-	int rc;
+	unsigned int key;
+	QNode *q_node;
 
 	if (gid == -1) {
 		snprintf(buf, size, "unset");
 		return buf;
+	} else if (gid == 0) {
+		snprintf(buf, size, "root");
+		return buf;
 	}
 
 	// Check the cache first
-	if (gid_list_created == 0) {
-		nvpair_create(&gid_nvl);
-		nvpair_clear(&gid_nvl);
-		gid_list_created = 1;
+	if (gid_cache_created == 0) {
+		gid_cache = init_lru(19, NULL, "gid");
+		gid_cache_created = 1;
 	}
-	rc = nvpair_find_val(&gid_nvl, gid);
-	if (rc) {
-		name = gid_nvl.cur->name;
-	} else {
-		// Add it to cache
-		struct group *gr;
-		gr = getgrgid(gid);
-		if (gr) {
-			nvpnode nv;
-			nv.name = strdup(gr->gr_name);
-			nv.val = gid;
-			nvpair_append(&gid_nvl, &nv);
-			name = gid_nvl.cur->name;
+	key = compute_subject_key(gid_cache, gid);
+	q_node = check_lru_cache(gid_cache, key);
+	if (q_node) {
+		if (q_node->id == gid)
+			name = q_node->str;
+		else {
+			// Add it to cache
+			struct group *gr;
+			lru_evict(gid_cache, key);
+			q_node = check_lru_cache(gid_cache, key);
+			gr = getgrgid(gid);
+			if (gr) {
+				q_node->str = strdup(gr->gr_name);
+				q_node->id = gid;
+				name = q_node->str;
+			}
 		}
 	}
 	if (name != NULL)
@@ -581,11 +585,11 @@ static const char *aulookup_gid(gid_t gid, char *buf, size_t size)
 
 void aulookup_destroy_gid_list(void)
 {
-	if (gid_list_created == 0)
+	if (gid_cache_created == 0)
 		return;
 
-	nvpair_clear(&gid_nvl); 
-	gid_list_created = 0;
+	destroy_lru(gid_cache); 
+	gid_cache_created = 0;
 }
 
 static const char *print_uid(const char *val, unsigned int base)
@@ -2703,7 +2707,7 @@ int lookup_type(const char *name)
  * This is the main entry point for the auparse library. Call chain is:
  * auparse_interpret_field -> nvlist_interp_cur_val -> interpret
  */
-const char *interpret(const rnode *r)
+const char *interpret(const rnode *r, auparse_esc_t escape_mode)
 {
 	const nvlist *nv = &r->nv;
 	int type;
@@ -2719,7 +2723,7 @@ const char *interpret(const rnode *r)
 	id.val = nvlist_get_cur_val(nv);
 	type = auparse_interp_adjust_type(r->type, id.name, id.val);
 
-	out = auparse_do_interpretation(type, &id);
+	out = auparse_do_interpretation(type, &id, escape_mode);
 	n = nvlist_get_cur(nv);
 	n->interp_val = (char *)out;
 
@@ -2775,7 +2779,8 @@ int auparse_interp_adjust_type(int rtype, const char *name, const char *val)
  * This can be called by either interpret() or from ausearch-report or
  * auditctl-listing.c. Returns a malloc'ed buffer that the caller must free.
  */
-char *auparse_do_interpretation(int type, const idata *id)
+char *auparse_do_interpretation(int type, const idata *id,
+	auparse_esc_t escape_mode)
 {
 	const char *out;
 
@@ -2922,11 +2927,11 @@ char *auparse_do_interpretation(int type, const idata *id)
 		}
 		if (str == NULL) {
 			// This is the normal path
-			unsigned int cnt = need_escaping(out, len);
+			unsigned int cnt = need_escaping(out, len, escape_mode);
 			if (cnt) {
 				char *dest = malloc(len + 1 + (3*cnt));
 				if (dest)
-					escape(out, dest, len);
+					escape(out, dest, len, escape_mode);
 				free((void *)out);
 				out = dest;
 			}
@@ -2939,7 +2944,7 @@ char *auparse_do_interpretation(int type, const idata *id)
 				unsigned int klen = str - ptr;
 				char tmp = *str;
 				*str = 0;
-				cnt += need_escaping(ptr, klen);
+				cnt += need_escaping(ptr, klen, escape_mode);
 				*str = tmp;
 				ptr = str;
 				// If we are not at the end...
@@ -2958,7 +2963,7 @@ char *auparse_do_interpretation(int type, const idata *id)
 				// actually put a control character in a key.
 				char *dest = malloc(len + 1 + (3*cnt));
 				if (dest)
-					key_escape(out, dest);
+					key_escape(out, dest, escape_mode);
 				free((void *)out);
 				out = dest;
 			}
