@@ -1,5 +1,5 @@
 /* libaudit.c -- 
- * Copyright 2004-2009,2012,2014,2016 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2004-2009,2012,2014,2016-17 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -513,6 +513,25 @@ int audit_set_backlog_wait_time(int fd, uint32_t bwt)
 	return rc;
 }
 
+int audit_reset_lost(int fd)
+{
+	int rc;
+	struct audit_status s;
+
+	if ((audit_get_features() & AUDIT_FEATURE_BITMAP_LOST_RESET) == 0)
+		return -30;
+
+	memset(&s, 0, sizeof(s));
+	s.mask = AUDIT_STATUS_LOST;
+	s.lost = 0;
+	rc = audit_send(fd, AUDIT_SET, &s, sizeof(s));
+	if (rc < 0)
+		audit_msg(audit_priority(errno),
+			"Error sending lost reset request (%s)", 
+			strerror(-rc));
+	return rc;
+}
+
 int audit_set_feature(int fd, unsigned feature, unsigned value, unsigned lock)
 {
 #if defined(HAVE_DECL_AUDIT_FEATURE_VERSION) && \
@@ -738,10 +757,6 @@ int audit_add_rule_data(int fd, struct audit_rule_data *rule,
 {
 	int rc;
 
-	if (flags == AUDIT_FILTER_ENTRY) {
-		audit_msg(LOG_WARNING, "Use of entry filter is deprecated");
-		return -2;
-	}
 	rule->flags  = flags;
 	rule->action = action;
 	rc = audit_send(fd, AUDIT_ADD_RULE, rule, 
@@ -759,10 +774,6 @@ int audit_delete_rule_data(int fd, struct audit_rule_data *rule,
 {
 	int rc;
 
-	if (flags == AUDIT_FILTER_ENTRY) {
-		audit_msg(LOG_WARNING, "Use of entry filter is deprecated");
-		return -2;
-	}
 	rule->flags  = flags;
 	rule->action = action;
 	rc = audit_send(fd, AUDIT_DEL_RULE, rule, 
@@ -886,12 +897,41 @@ int audit_setloginuid(uid_t uid)
 	return rc;
 }
 
+/*
+ * This function will retreive the login session or -2 if there
+ * is an error.
+ */
+uint32_t audit_get_session(void)
+{
+	uint32_t ses;
+	int len, in;
+	char buf[16];
+
+	errno = 0;
+	in = open("/proc/self/sessionid", O_NOFOLLOW|O_RDONLY);
+	if (in < 0)
+		return -2;
+	do {
+		len = read(in, buf, sizeof(buf));
+	} while (len < 0 && errno == EINTR);
+	close(in);
+	if (len < 0 || len >= sizeof(buf))
+		return -2;
+	buf[len] = 0;
+	errno = 0;
+	ses = strtoul(buf, 0, 10);
+	if (errno)
+		return -2;
+	else
+		return ses;
+}
+
 int audit_rule_syscall_data(struct audit_rule_data *rule, int scall)
 {
 	int word = AUDIT_WORD(scall);
 	int bit  = AUDIT_BIT(scall);
 
-	if (word >= (AUDIT_BITMASK_SIZE-1)) 
+	if (word > (AUDIT_BITMASK_SIZE-1)) 
 		return -1;
 	rule->mask[word] |= bit;
 	return 0;
@@ -904,7 +944,7 @@ int audit_rule_syscallbyname_data(struct audit_rule_data *rule,
 	int machine;
 
 	if (!strcmp(scall, "all")) {
-		for (i = 0; i < (AUDIT_BITMASK_SIZE-1); i++) 
+		for (i = 0; i < AUDIT_BITMASK_SIZE; i++) 
 			rule->mask[i] = ~0;
 		return 0;
 	}
@@ -1399,9 +1439,30 @@ int audit_rule_fieldpair_data(struct audit_rule_data **rulep, const char *pair,
 	if ((field = audit_name_to_field(f)) < 0) 
 		return -2;
 
-	/* Exclude filter can be used only with MSGTYPE field */
-	if (flags == AUDIT_FILTER_EXCLUDE && field != AUDIT_MSGTYPE)
-		return -12; 
+	/* Exclude filter can be used only with MSGTYPE and cred fields */
+	if (flags == AUDIT_FILTER_EXCLUDE) {
+		uint32_t features = audit_get_features();
+		if ((features & AUDIT_FEATURE_BITMAP_EXCLUDE_EXTEND) == 0) {
+			if (field != AUDIT_MSGTYPE)
+				return -30;
+		} else {
+			switch(field) {
+				case AUDIT_PID:
+				case AUDIT_UID:
+				case AUDIT_GID:
+				case AUDIT_LOGINUID:
+				case AUDIT_MSGTYPE:
+				case AUDIT_SUBJ_USER:
+				case AUDIT_SUBJ_ROLE:
+				case AUDIT_SUBJ_TYPE:
+				case AUDIT_SUBJ_SEN:
+				case AUDIT_SUBJ_CLR:
+					break;
+				default:
+					return -12;
+			}
+		}
+	}
 
 	rule->fields[rule->field_count] = field;
 	rule->fieldflags[rule->field_count] = op;
@@ -1606,8 +1667,7 @@ int audit_rule_fieldpair_data(struct audit_rule_data **rulep, const char *pair,
 			}
 			break;
 		case AUDIT_FILETYPE:
-			if (!(flags == AUDIT_FILTER_EXIT ||
-						flags == AUDIT_FILTER_ENTRY))
+			if (!(flags == AUDIT_FILTER_EXIT))
 				return -17;
 			rule->values[rule->field_count] = 
 				audit_name_to_ftype(v);
@@ -1627,6 +1687,26 @@ int audit_rule_fieldpair_data(struct audit_rule_data **rulep, const char *pair,
 			else 
 				return -21;
 			break;
+		case AUDIT_SESSIONID:
+			if ((audit_get_features() &
+				AUDIT_FEATURE_BITMAP_SESSIONID_FILTER) == 0)
+				return -30;
+			if (flags != AUDIT_FILTER_EXCLUDE &&
+			    flags != AUDIT_FILTER_USER &&
+			    flags != AUDIT_FILTER_EXIT)
+				return -31;
+			// Do positive & negative separate for 32 bit systems
+			vlen = strlen(v);
+			if (isdigit((char)*(v))) 
+				rule->values[rule->field_count] = 
+					strtoul(v, NULL, 0);
+			else if (vlen >= 2 && *(v)=='-' &&
+						(isdigit((char)*(v+1))))
+				rule->values[rule->field_count] =
+					strtol(v, NULL, 0);
+			else if (strcmp(v, "unset") == 0)
+				rule->values[rule->field_count] = 4294967295;
+			break;
 		case AUDIT_DEVMAJOR...AUDIT_INODE:
 		case AUDIT_SUCCESS:
 			if (flags != AUDIT_FILTER_EXIT)
@@ -1639,8 +1719,7 @@ int audit_rule_fieldpair_data(struct audit_rule_data **rulep, const char *pair,
 					return -13;
 			}
 
-			if (field == AUDIT_PPID && !(flags == AUDIT_FILTER_EXIT
-				|| flags == AUDIT_FILTER_ENTRY))
+			if (field == AUDIT_PPID && !(flags==AUDIT_FILTER_EXIT))
 				return -17;
 			
 			if (!isdigit((char)*(v)))
