@@ -1,5 +1,5 @@
 /* libaudit.h -- 
- * Copyright 2004-2016 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2004-2017 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -268,6 +268,11 @@ extern "C" {
 #define AUDIT_FILTER_MASK	0x07	/* Mask to get actual filter */
 #define AUDIT_FILTER_UNSET	0x80	/* This value means filter is unset */
 
+/* Status symbol mask values */
+#ifndef AUDIT_STATUS_LOST
+#define AUDIT_STATUS_LOST               0x0040
+#endif
+
 /* These defines describe what features are in the kernel */
 #ifndef AUDIT_FEATURE_BITMAP_BACKLOG_LIMIT
 #define AUDIT_FEATURE_BITMAP_BACKLOG_LIMIT      0x00000001
@@ -278,6 +283,15 @@ extern "C" {
 #ifndef AUDIT_FEATURE_BITMAP_EXECUTABLE_PATH
 #define AUDIT_FEATURE_BITMAP_EXECUTABLE_PATH    0x00000004
 #endif
+#ifndef AUDIT_FEATURE_BITMAP_EXCLUDE_EXTEND
+#define AUDIT_FEATURE_BITMAP_EXCLUDE_EXTEND     0x00000008
+#endif
+#ifndef AUDIT_FEATURE_BITMAP_SESSIONID_FILTER
+#define AUDIT_FEATURE_BITMAP_SESSIONID_FILTER   0x00000010
+#endif
+#ifndef AUDIT_FEATURE_BITMAP_LOST_RESET
+#define AUDIT_FEATURE_BITMAP_LOST_RESET		0x00000020
+#endif
 
 /* Defines for interfield comparison update */
 #ifndef AUDIT_OBJ_UID
@@ -293,6 +307,10 @@ extern "C" {
 #define AUDIT_EXE 112
 #endif
 
+#ifndef AUDIT_SESSIONID
+#define AUDIT_SESSIONID 25
+#endif
+
 #ifndef AUDIT_COMPARE_UID_TO_OBJ_UID
 #define AUDIT_COMPARE_UID_TO_OBJ_UID   1
 #endif
@@ -384,6 +402,9 @@ extern "C" {
 #define AUDIT_ARCH_PPC64LE	(EM_PPC64|__AUDIT_ARCH_64BIT|__AUDIT_ARCH_LE)
 #endif
 
+/* This is the character that separates event data from enrichment fields */
+#define AUDIT_INTERP_SEPARATOR 0x1D
+
 //////////////////////////////////////////////////////
 // This is an external ABI. Any changes in here will
 // likely affect pam_loginuid. There might be other
@@ -423,7 +444,7 @@ struct audit_reply {
 	struct nlmsgerr         *error;
 	struct audit_sig_info   *signal_info;
 	struct daemon_conf      *conf;
-#if HAVE_DECL_AUDIT_FEATURE_VERSION
+#ifdef AUDIT_FEATURE_BITMAP_ALL
 	struct audit_features	*features;
 #endif
 	};
@@ -448,7 +469,13 @@ struct audit_dispatcher_header {
 	uint32_t	size;	/* Size of data following the header */
 };
 
-#define AUDISP_PROTOCOL_VER 0
+// Original protocol starts with msg='
+#define AUDISP_PROTOCOL_VER  0
+
+// Starts with node and/or type already in the text before msg=
+// IOW, its preformatted in the audit daemon.
+#define AUDISP_PROTOCOL_VER2 1
+
 
 ///////////////////////////////////////////////////
 // Libaudit API
@@ -489,6 +516,7 @@ extern int  audit_get_reply(int fd, struct audit_reply *rep, reply_t block,
 		int peek);
 extern uid_t audit_getloginuid(void);
 extern int  audit_setloginuid(uid_t uid);
+extern uint32_t audit_get_session(void);
 extern int  audit_detect_machine(void);
 extern int audit_determine_machine(const char *arch);
 
@@ -529,6 +557,7 @@ extern int  audit_set_failure(int fd, uint32_t failure);
 extern int  audit_set_rate_limit(int fd, uint32_t limit);
 extern int  audit_set_backlog_limit(int fd, uint32_t limit);
 int audit_set_backlog_wait_time(int fd, uint32_t bwt);
+int audit_reset_lost(int fd);
 extern int  audit_set_feature(int fd, unsigned feature, unsigned value, unsigned lock);
 extern int  audit_set_loginuid_immutable(int fd);
 
@@ -594,6 +623,11 @@ extern int audit_rule_interfield_comp_data(struct audit_rule_data **rulep,
 					 const char *pair, int flags);
 extern void audit_rule_free_data(struct audit_rule_data *rule);
 
+/* Capability testing functions */
+int audit_can_control(void);
+int audit_can_write(void);
+int audit_can_read(void);
+
 #ifdef __cplusplus
 }
 #endif
