@@ -1,5 +1,5 @@
 /* auditctl.c -- 
- * Copyright 2004-2016 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2004-2017 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -131,10 +131,13 @@ static void usage(void)
      "    -W <path>           Remove watch at <path>\n"
 #if defined(HAVE_DECL_AUDIT_FEATURE_VERSION) && \
     defined(HAVE_STRUCT_AUDIT_STATUS_FEATURE_BITMAP)
-     "    --loginuid-immutable   Make loginuids unchangeable once set\n"
+     "    --loginuid-immutable  Make loginuids unchangeable once set\n"
 #endif
 #if HAVE_DECL_AUDIT_VERSION_BACKLOG_WAIT_TIME
-     "    --backlog_wait_time    Set the kernel backlog_wait_time\n"
+     "    --backlog_wait_time  Set the kernel backlog_wait_time\n"
+#endif
+#if defined(HAVE_STRUCT_AUDIT_STATUS_FEATURE_BITMAP)
+     "    --reset-lost         Reset the lost record counter\n"
 #endif
      );
 }
@@ -143,8 +146,6 @@ static int lookup_filter(const char *str, int *filter)
 {
 	if (strcmp(str, "task") == 0) 
 		*filter = AUDIT_FILTER_TASK;
-	else if (strcmp(str, "entry") == 0)
-		*filter = AUDIT_FILTER_ENTRY;
 	else if (strcmp(str, "exit") == 0)
 		*filter = AUDIT_FILTER_EXIT;
 	else if (strcmp(str, "user") == 0)
@@ -213,16 +214,6 @@ static int audit_rule_setup(char *opt, int *filter, int *act, int lineno)
 	if (*filter == AUDIT_FILTER_UNSET || *act == -1)
 		return 2;
 
-	/* Consolidate rules on exit filter */
-	if (*filter == AUDIT_FILTER_ENTRY) {
-		*filter = AUDIT_FILTER_EXIT;
-		if (lineno)
-			audit_msg(LOG_INFO, "Warning - entry rules deprecated, changing to exit rule in line %d", lineno);
-		else
-			audit_msg(LOG_INFO,
-		    "Warning - entry rules deprecated, changing to exit rule");
-	}
-
 	return 0;
 }
 
@@ -523,6 +514,7 @@ struct option long_opts[] =
 #endif
 #if HAVE_DECL_AUDIT_VERSION_BACKLOG_WAIT_TIME
   {"backlog_wait_time", 1, NULL, 2},
+  {"reset-lost", 0, NULL, 3},
 #endif
   {NULL, 0, NULL, 0}
 };
@@ -534,7 +526,7 @@ struct option long_opts[] =
  */
 static int setopt(int count, int lineno, char *vars[])
 {
-    int c;
+    int c, lidx = 0;
     int retval = 0, rc;
 
     optind = 0;
@@ -544,7 +536,7 @@ static int setopt(int count, int lineno, char *vars[])
 
     while ((retval >= 0) && (c = getopt_long(count, vars,
 			"hicslDvtC:e:f:r:b:a:A:d:S:F:m:R:w:W:k:p:q:",
-			long_opts, NULL)) != EOF) {
+			long_opts, &lidx)) != EOF) {
 	int flags = AUDIT_FILTER_UNSET;
 	rc = 10;	// Init to something impossible to see if unused.
         switch (c) {
@@ -1052,6 +1044,15 @@ process_keys:
 		retval = -1;
 #endif
 		break;
+	case 3:
+		if ((rc = audit_reset_lost(fd)) >= 0) {
+			audit_msg(LOG_INFO, "lost: %u", rc);
+			return -2;
+		} else {
+			audit_number_to_errmsg(rc, long_opts[lidx].name);
+			retval = -1;
+		}
+		break;
         default: 
 		usage();
 		retval = -1;
@@ -1321,7 +1322,9 @@ int main(int argc, char *argv[])
 #ifndef DEBUG
 	/* Make sure we are root if we do anything except help */
 	if (!(argc == 2 && (strcmp(argv[1], "--help")==0 ||
-			strcmp(argv[1], "-h") == 0)) && !audit_can_control()) {
+			strcmp(argv[1], "-h") == 0 ||
+			(strcmp(argv[1], "-l") == 0 && geteuid() == 0))) &&
+			!audit_can_control()) {
 		audit_msg(LOG_WARNING, "You must be root to run this program.");
 		return 4;
 	}
