@@ -48,6 +48,8 @@
 extern volatile int stop;
 
 /* Local function prototypes */
+static void send_ack(const struct auditd_event *e, int ack_type,
+			const char *msg);
 static void write_to_log(const struct auditd_event *e);
 static void check_log_file_size(void);
 static void check_space_left(void);
@@ -104,6 +106,7 @@ void shutdown_events(void)
 
 	free((void *)format_buf);
 	fclose(log_file);
+	auparse_destroy_ext(NULL, AUPARSE_DESTROY_ALL);
 }
 
 int init_event(struct daemon_conf *conf)
@@ -379,7 +382,7 @@ static const char *format_enrich(const struct audit_reply *rep)
 			free(message);
 			return format_buf;
 		}
-		auparse_set_escape_mode(NULL, AUPARSE_ESC_RAW);
+		auparse_set_escape_mode(au, AUPARSE_ESC_RAW);
 		sep_done = 0;
 
 		// Loop over all fields while possible to add field
@@ -411,7 +414,7 @@ static const char *format_enrich(const struct audit_reply *rep)
 			rc = auparse_next_field(au);
 		}
 
-		auparse_destroy(au);
+		auparse_destroy_ext(au, AUPARSE_DESTROY_COMMON);
 		free(message);
 	}
         return format_buf;
@@ -440,7 +443,10 @@ void format_event(struct auditd_event *e)
 /* This function free's all memory associated with events */
 void cleanup_event(struct auditd_event *e)
 {
-	free((void *)e->reply.message);
+	// Over in send_audit_event we sometimes have message pointing
+	// into the middle of the reply allocation. Check for it.
+	if (e->reply.message != e->reply.msg.data)
+		free((void *)e->reply.message);
 	free(e);
 }
 
@@ -493,7 +499,7 @@ void handle_event(struct auditd_event *e)
 		if (config->write_logs == 0)
 			return;
 	}
-	if (!logging_suspended) {
+	if (!logging_suspended && config->write_logs) {
 		write_to_log(e);
 
 		/* See if we need to flush to disk manually */
@@ -534,7 +540,10 @@ void handle_event(struct auditd_event *e)
 				}
 			}
 		}
-	}
+	} else if (!config->write_logs)
+		send_ack(e, AUDIT_RMW_TYPE_ACK, "");
+	// FIXME: When logging is suspended, what should remote do?
+	// Should probably be new response type
 }
 
 static void send_ack(const struct auditd_event *e, int ack_type,
@@ -1490,6 +1499,9 @@ static void reconfigure(struct auditd_event *e)
 			logging_suspended = saved_suspend;
 	}
 
+	/* This had to wait until now so the child exec has happened */
+	make_dispatcher_fd_private();
+
 	// Next document the results
 	srand(time(NULL));
 	seq_num = rand()%10000;
