@@ -352,6 +352,8 @@ static int parse_task_info(lnode *n, search_items *s)
 			if (term == NULL)
 				return 34;
 			*term = 0;
+			if (s->terminal) // ANOM_NETLINK has one
+				free(s->terminal);
 			s->terminal = strdup(str);
 			*term = ' ';
 		}
@@ -405,6 +407,8 @@ static int parse_task_info(lnode *n, search_items *s)
 				if (term == NULL)
 					return 39;
 				*term = 0;
+				if (s->exe) // ANOM_NETLINK has one
+					free(s->exe);
 				s->exe = strdup(str);
 				*term = '"';
 			} else 
@@ -824,7 +828,6 @@ static int parse_user(const lnode *n, search_items *s)
 	}
 	// optionally get loginuid
 	if (event_loginuid != -2 || event_tauid) {
-		*term = ' ';
 		str = strstr(term, "auid=");
 		if (str == NULL) { // Try the older one
 			str = strstr(term, "loginuid=");
@@ -1390,6 +1393,46 @@ static int parse_daemon1(const lnode *n, search_items *s)
 		*term = saved;
 	}
 
+	// uid - optional
+	if (event_uid != -1) {
+		ptr = term;
+		str = strstr(term, " uid=");
+		if (str) {
+			ptr = str + 5;
+			term = strchr(ptr, ' ');
+			if (term == NULL) 
+				return 7;
+			saved = *term;
+			*term = 0;
+			errno = 0;
+			s->uid = strtoul(ptr, NULL, 10);
+			if (errno)
+				return 8;
+			*term = saved;
+		} else
+			term = ptr;
+	}
+
+	// ses - optional
+	if (event_session_id != -2) {
+		ptr = term;
+		str = strstr(term, "ses=");
+		if (str) {
+			ptr = str + 4;
+			term = strchr(ptr, ' ');
+			if (term == NULL) 
+				return 9;
+			saved = *term;
+			*term = 0;
+			errno = 0;
+			s->session_id = strtoul(ptr, NULL, 10);
+			if (errno)
+				return 10;
+			*term = saved;
+		} else
+			term = ptr;
+	}
+
 	if (event_subject) {
 		// scontext
 		str = strstr(term, "subj=");
@@ -1405,7 +1448,7 @@ static int parse_daemon1(const lnode *n, search_items *s)
 				an.scontext = strdup(str);
 				alist_append(s->avc, &an);
 			} else
-				return 7;
+				return 11;
 			if (term)
 				*term = ' ';
 		}
@@ -1419,7 +1462,7 @@ static int parse_daemon1(const lnode *n, search_items *s)
 			while (isalpha(*term))
 				term++;
 			if (term == ptr)
-				return 9;
+				return 12;
 			saved = *term;
 			*term = 0;
 			if (strncmp(ptr, "failed", 6) == 0)
