@@ -32,6 +32,7 @@
 #include "ausearch-lookup.h"
 #include "auparse-idata.h"
 #include "auparse-defs.h"
+#include "auditd-config.h"
 
 /* Local functions */
 static void output_raw(llist *l);
@@ -99,7 +100,9 @@ static void output_raw(llist *l)
 		return;
 	}
 	do {
-		n->message[n->mlen] = AUDIT_INTERP_SEPARATOR;
+		// Only add the separator for enriched events.
+		if (l->fmt == LF_ENRICHED)
+			n->message[n->mlen] = AUDIT_INTERP_SEPARATOR;
 		puts(n->message);
 	} while ((n=list_next(l)));
 }
@@ -356,7 +359,7 @@ static void interpret(char *name, char *val, int comma, int rtype)
 	id.name = name;
 	id.val = val;
 
-	char *out = auparse_do_interpretation(type, &id);
+	char *out = auparse_do_interpretation(type, &id, AUPARSE_ESC_TTY);
 	if (type == AUPARSE_TYPE_UNCLASSIFIED)
 		printf("%s%c", val, comma ? ',' : ' ');
 	else if (name[0] == 'k' && strcmp(name, "key") == 0) {
