@@ -31,7 +31,6 @@
 #include <regex.h>
 #include <time.h>
 #include <sys/types.h>
-#include <pwd.h>
 #include <unistd.h>
 #include "auparse.h"
 #include "libaudit.h"
@@ -73,7 +72,7 @@ struct event {
 	enum event_type type;
 	time_t start;
 	time_t end;
-	uid_t uid;
+	char *user;
 	char *uuid;
 	char *name;
 	int success;
@@ -106,12 +105,11 @@ struct event *event_alloc(void)
 	struct event *event = malloc(sizeof(struct event));
 	if (event) {
 		/* The new event is initialized with values that represents
-		 * unset values: -1 for uid and pid and 0 (or NULL) for numbers
+		 * unset values: -1 for user and pid and 0 (or NULL) for numbers
 		 * and pointers. For example, event->end = 0 represents an
 		 * unfinished event.
 		 */
 		memset(event, 0, sizeof(struct event));
-		event->uid = -1;
 		event->pid = -1;
 	}
 	return event;
@@ -120,6 +118,7 @@ struct event *event_alloc(void)
 void event_free(struct event *event)
 {
 	if (event) {
+		free(event->user);
 		free(event->uuid);
 		free(event->name);
 		free(event->reason);
@@ -289,8 +288,8 @@ auparse_state_t *init_auparse(void)
 		au = auparse_init(AUSOURCE_FILE, file);
 	} else {
 		if (getuid()) {
-			fprintf(stderr, "You probably need to be root for "
-					"this to work\n");
+			fprintf(stderr,
+			   "You probably need to be root for this to work\n");
 		}
 		au = auparse_init(AUSOURCE_LOGS, NULL);
 	}
@@ -369,16 +368,20 @@ int create_search_criteria(auparse_state_t *au)
 
 /* Extract the most common fields from virtualization-related records. */
 int extract_virt_fields(auparse_state_t *au, const char **p_uuid,
-		uid_t *p_uid, time_t *p_time, const char **p_name,
+		const char **p_user, time_t *p_time, const char **p_name,
 		int *p_suc)
 {
-	const char *field;
+	const char *field = NULL;
 	auparse_first_record(au);
+
 	/* Order matters */
-	if (p_uid) {
+	if (p_user) {
+		char *t;
 		if (!auparse_find_field(au, field = "uid"))
 			goto error;
-		*p_uid = auparse_get_field_int(au);
+		t = auparse_interpret_field(au);
+		if (t)
+			*p_user = strdup(t);
 	}
 	if (p_name) {
 		if (!auparse_find_field(au, field = "vm"))
@@ -409,6 +412,8 @@ error:
 				auparse_get_milli(au),
 				auparse_get_serial(au));
 	}
+	if (p_user)
+		free(*p_user);
 	return 1;
 }
 
@@ -459,9 +464,8 @@ int add_proof(struct event *event, auparse_state_t *au)
  */
 int process_machine_id_event(auparse_state_t *au)
 {
-	uid_t uid;
 	time_t time;
-	const char *seclevel, *uuid, *name;
+	const char *seclevel, *uuid, *name, *user = NULL;
 	struct event *event;
 	int success;
 
@@ -472,7 +476,7 @@ int process_machine_id_event(auparse_state_t *au)
 					"MACHINE_ID event.\n");
 	}
 
-	if (extract_virt_fields(au, &uuid, &uid, &time, &name, &success))
+	if (extract_virt_fields(au, &uuid, &user, &time, &name, &success))
 		return 0;
 
 	event = event_alloc();
@@ -483,7 +487,7 @@ int process_machine_id_event(auparse_state_t *au)
 	event->name = copy_str(name);
 	event->success = success;
 	event->seclevel = copy_str(seclevel);
-	event->uid = uid;
+	event->user = user;
 	event->start = time;
 	add_proof(event, au);
 	if (list_append(events, event) == NULL) {
@@ -496,14 +500,13 @@ int process_machine_id_event(auparse_state_t *au)
 int add_start_guest_event(auparse_state_t *au)
 {
 	struct event *start;
-	uid_t uid;
 	time_t time;
-	const char *uuid, *name;
+	const char *uuid, *name, *user = NULL;
 	int success;
 	list_node_t *it;
 
 	/* Just skip this record if it failed to get some of the fields */
-	if (extract_virt_fields(au, &uuid, &uid, &time, &name, &success))
+	if (extract_virt_fields(au, &uuid, &user, &time, &name, &success))
 		return 0;
 
 	/* On failure, loop backwards to update all the resources associated to
@@ -531,13 +534,15 @@ int add_start_guest_event(auparse_state_t *au)
 	}
 
 	start = event_alloc();
-	if (start == NULL)
+	if (start == NULL) {
+		free(user);
 		return 1;
+	}
 	start->type = ET_START;
 	start->uuid = copy_str(uuid);
 	start->name = copy_str(name);
 	start->success = success;
-	start->uid = uid;
+	start->user = user;
 	start->start = time;
 	auparse_first_record(au);
 	if (auparse_find_field(au, "vm-pid"))
@@ -554,13 +559,12 @@ int add_stop_guest_event(auparse_state_t *au)
 {
 	list_node_t *it;
 	struct event *stop, *start = NULL, *event = NULL;
-	uid_t uid;
 	time_t time;
-	const char *uuid, *name;
+	const char *uuid, *name, *user = NULL;
 	int success;
 
 	/* Just skip this record if it failed to get some of the fields */
-	if (extract_virt_fields(au, &uuid, &uid, &time, &name, &success))
+	if (extract_virt_fields(au, &uuid, &user, &time, &name, &success))
 		return 0;
 
 	/* Loop backwards to find the last start event for the uuid and
@@ -595,24 +599,28 @@ int add_stop_guest_event(auparse_state_t *au)
 			fprintf(stderr, "Couldn't find the correlated start "
 					"record to the stop event.\n");
 		}
+		free(user);
 		return 0;
 	}
 
 	/* Create a new stop event */
 	stop = event_alloc();
-	if (stop == NULL)
+	if (stop == NULL) {
+		free(user);
 		return 1;
+	}
 	stop->type = ET_STOP;
 	stop->uuid = copy_str(uuid);
 	stop->name = copy_str(name);
 	stop->success = success;
-	stop->uid = uid;
+	stop->user = user;
 	stop->start = time;
 	auparse_first_record(au);
 	if (auparse_find_field(au, "vm-pid"))
 		stop->pid = auparse_get_field_int(au);
 	add_proof(stop, au);
 	if (list_append(events, stop) == NULL) {
+		free(user);
 		event_free(stop);
 		return 1;
 	}
@@ -658,12 +666,14 @@ static int is_resource(const char *res)
 	return 1;
 }
 
-int add_resource(auparse_state_t *au, const char *uuid, uid_t uid, time_t time,
-		const char *name, int success, const char *reason,
+int add_resource(auparse_state_t *au, const char *uuid, const char *user,
+		time_t time, const char *name, int success, const char *reason,
 		const char *res_type, const char *res)
 {
-	if (!is_resource(res))
+	if (!is_resource(res)) {
+		free(user);
 		return 0;
+	}
 
 	struct event *event = event_alloc();
 	if (event == NULL)
@@ -675,7 +685,7 @@ int add_resource(auparse_state_t *au, const char *uuid, uid_t uid, time_t time,
 	event->reason = copy_str(reason);
 	event->res_type = copy_str(res_type);
 	event->res = copy_str(res);
-	event->uid = uid;
+	event->user = user;
 	event->start = time;
 	add_proof(event, au);
 
@@ -702,8 +712,8 @@ int add_resource(auparse_state_t *au, const char *uuid, uid_t uid, time_t time,
 	return 0;
 }
 
-int update_resource(auparse_state_t *au, const char *uuid, uid_t uid,
-		time_t time, const char *name, int success, const char *reason,
+int update_resource(auparse_state_t *au, const char *uuid, time_t time,
+		const char *name, int success, const char *reason,
 		const char *res_type, const char *res)
 {
 	if (!is_resource(res) || !success)
@@ -738,15 +748,14 @@ int update_resource(auparse_state_t *au, const char *uuid, uid_t uid,
 
 int process_resource_event(auparse_state_t *au)
 {
-	uid_t uid;
 	time_t time;
-	const char *res_type, *uuid, *name;
+	const char *res_type, *uuid, *name, *user = NULL;
 	char field[64];
 	const char *reason;
 	int success;
 
 	/* Just skip this record if it failed to get some of the fields */
-	if (extract_virt_fields(au, &uuid, &uid, &time, &name, &success))
+	if (extract_virt_fields(au, &uuid, &user, &time, &name, &success))
 		return 0;
 
 	/* Get the resource type */
@@ -756,6 +765,7 @@ int process_resource_event(auparse_state_t *au)
 	if (res_type == NULL) {
 		if (debug)
 			fprintf(stderr, "Invalid resrc field.\n");
+		free(user);
 		return 0;
 	}
 
@@ -778,7 +788,7 @@ int process_resource_event(auparse_state_t *au)
 		if (res == NULL && debug) {
 			fprintf(stderr, "Failed to get %s field.\n", field);
 		} else {
-			rc += update_resource(au, uuid, uid, time, name,
+			rc += update_resource(au, uuid, time, name,
 					success, reason, res_type, res);
 		}
 
@@ -789,8 +799,9 @@ int process_resource_event(auparse_state_t *au)
 			res = auparse_interpret_field(au);
 		if (res == NULL && debug) {
 			fprintf(stderr, "Failed to get %s field.\n", field);
+			free(user);
 		} else {
-			rc += add_resource(au, uuid, uid, time, name, success,
+			rc += add_resource(au, uuid, user, time, name, success,
 					reason, res_type, res);
 		}
 	} else if (strcmp("cgroup", res_type) == 0) {
@@ -798,12 +809,14 @@ int process_resource_event(auparse_state_t *au)
 		const char *cgroup = NULL;
 		if (auparse_find_field(au, "cgroup"))
 			cgroup = auparse_interpret_field(au);
-		rc += add_resource(au, uuid, uid, time, name, success, reason,
+		rc += add_resource(au, uuid, user, time, name, success, reason,
 				res_type, cgroup);
 	} else if (debug) {
 		fprintf(stderr, "Found an unknown resource: %s.\n",
 				res_type);
-	}
+		free(user);
+	} else
+		free(user);
 	return rc;
 }
 
@@ -828,9 +841,8 @@ struct event *get_machine_id_by_seclevel(const char *seclevel)
 
 int process_avc_selinux_context(auparse_state_t *au, const char *context)
 {
-	const char *seclevel;
+	const char *seclevel, *user = NULL;
 	struct event *machine_id, *avc;
-	uid_t uid;
 	time_t time;
 
 	seclevel = get_seclevel(auparse_find_field(au, context));
@@ -842,7 +854,7 @@ int process_avc_selinux_context(auparse_state_t *au, const char *context)
 		return 0;
 	}
 
-	if (extract_virt_fields(au, NULL, &uid, &time, NULL, NULL))
+	if (extract_virt_fields(au, NULL, &user, &time, NULL, NULL))
 		return 0;
 
 	machine_id = get_machine_id_by_seclevel(seclevel);
@@ -866,7 +878,7 @@ int process_avc_selinux_context(auparse_state_t *au, const char *context)
 
 	/* AVC info */
 	avc->start = time;
-	avc->uid = uid;
+	avc->user = user;
 	avc->seclevel = copy_str(seclevel);
 	auparse_first_record(au);
 	avc->avc_result = copy_str(auparse_find_field(au, "seresult"));
@@ -911,10 +923,9 @@ int process_avc_selinux(auparse_state_t *au)
 #ifdef WITH_APPARMOR
 int process_avc_apparmor_source(auparse_state_t *au)
 {
-	uid_t uid = -1;
 	time_t time = 0;
 	struct event *avc;
-	const char *target;
+	const char *target, *user = NULL;
 
 	/* Get the target object. */
 	if (auparse_find_field(au, "name") == NULL) {
@@ -959,12 +970,14 @@ int process_avc_apparmor_source(auparse_state_t *au)
 		return 0;
 	}
 
-	if (extract_virt_fields(au, NULL, &uid, &time, NULL, NULL))
+	if (extract_virt_fields(au, NULL, &user, &time, NULL, NULL))
 		return 0;
 
 	avc = event_alloc();
-	if (avc == NULL)
+	if (avc == NULL) {
+		free(user);
 		return 1;
+	}
 	avc->type = ET_AVC;
 
 	/* Guest info */
@@ -974,7 +987,7 @@ int process_avc_apparmor_source(auparse_state_t *au)
 
 	/* AVC info */
 	avc->start = time;
-	avc->uid = uid;
+	avc->user = user;
 	auparse_first_record(au);
 	if (auparse_find_field(au, "apparmor")) {
 		int i;
@@ -999,7 +1012,7 @@ int process_avc_apparmor_source(auparse_state_t *au)
 
 int process_avc_apparmor_target(auparse_state_t *au)
 {
-	uid_t uid;
+	const char *user = NULL;
 	time_t time;
 	const char *profile;
 	struct event *avc;
@@ -1068,12 +1081,14 @@ int process_avc_apparmor_target(auparse_state_t *au)
 		return 0;
 	}
 
-	if (extract_virt_fields(au, NULL, &uid, &time, NULL, NULL))
+	if (extract_virt_fields(au, NULL, &user, &time, NULL, NULL))
 		return 0;
 
 	avc = event_alloc();
-	if (avc == NULL)
+	if (avc == NULL) {
+		free(user);
 		return 1;
+	}
 	avc->type = ET_AVC;
 
 	/* Guest info */
@@ -1083,7 +1098,7 @@ int process_avc_apparmor_target(auparse_state_t *au)
 
 	/* AVC info */
 	avc->start = time;
-	avc->uid = uid;
+	avc->user = user;
 	auparse_first_record(au);
 	if (auparse_find_field(au, "apparmor")) {
 		int i;
@@ -1140,7 +1155,7 @@ int process_avc(auparse_state_t *au)
  * pid or the selinux context. */
 int process_anom(auparse_state_t *au)
 {
-	uid_t uid;
+	const char *user = NULL;
 	time_t time;
 	pid_t pid = -1;
 	list_node_t *it;
@@ -1222,16 +1237,18 @@ int process_anom(auparse_state_t *au)
 		return 0;
 	}
 
-	if (extract_virt_fields(au, NULL, &uid, &time, NULL, NULL))
+	if (extract_virt_fields(au, NULL, &user, &time, NULL, NULL))
 		return 0;
 
 	anom = event_alloc();
-	if (anom == NULL)
+	if (anom == NULL) {
+		free(user);
 		return 1;
+	}
 	anom->type = ET_ANOM;
 	anom->uuid = copy_str(start->uuid);
 	anom->name = copy_str(start->name);
-	anom->uid = uid;
+	anom->user = user;
 	anom->start = time;
 	anom->pid = pid;
 	memcpy(anom->proof, start->proof, sizeof(anom->proof));
@@ -1245,13 +1262,13 @@ int process_anom(auparse_state_t *au)
 
 int process_shutdown(auparse_state_t *au)
 {
-	uid_t uid = -1;
+	const char *user = NULL;
 	time_t time = 0;
 	struct event *down;
 	list_node_t *it;
 	int success = 0;
 
-	if (extract_virt_fields(au, NULL, &uid, &time, NULL, &success))
+	if (extract_virt_fields(au, NULL, &user, &time, NULL, &success))
 		return 0;
 
 	for (it = events->tail; it; it = it->prev) {
@@ -1269,10 +1286,12 @@ int process_shutdown(auparse_state_t *au)
 	}
 
 	down = event_alloc();
-	if (down == NULL)
+	if (down == NULL) {
+		free(user);
 		return 1;
+	}
 	down->type = ET_DOWN;
-	down->uid = uid;
+	down->user = user;
 	down->start = time;
 	down->success = success;
 	add_proof(down, au);
@@ -1286,7 +1305,7 @@ int process_shutdown(auparse_state_t *au)
 /* Convert record type to a string */
 const char *get_rec_type(struct event *e)
 {
-	static char buf[64];
+	static char buf[16];
 	if (e == NULL)
 		return "";
 
@@ -1309,24 +1328,6 @@ const char *get_rec_type(struct event *e)
 	return buf;
 }
 
-/* Convert uid to a string */
-const char *get_username(struct event *e)
-{
-	static char s[256];
-	if (!e || (int)e->uid == -1) {
-		s[0] = '?';
-		s[1] = '\0';
-	} else {
-		struct passwd *passwd = getpwuid(e->uid);
-		if (passwd == NULL || passwd->pw_name == NULL) {
-			snprintf(s, sizeof(s), "%d", e->uid);
-		} else {
-			snprintf(s, sizeof(s), "%s", passwd->pw_name);
-		}
-	}
-	return s;
-}
-
 /* Convert a time period to string */
 const char *get_time_period(struct event *event)
 {
@@ -1377,7 +1378,7 @@ void print_event(struct event *event)
 	printf("%-25.25s", N(event->name));
 	if (uuid_flag)
 		printf("\t%-36.36s", N(event->uuid));
-	printf("\t%-11.11s\t%-35.35s", get_username(event),
+	printf("\t%-11.11s\t%-35.35s", N(event->user),
 			get_time_period(event));
 
 	/* Print type specific fields */
